@@ -1,8 +1,8 @@
-## DNA.R (2019-02-22)
+## DNA.R (2021-12-16)
 
 ##   Manipulations and Comparisons of DNA and AA Sequences
 
-## Copyright 2002-2019 Emmanuel Paradis, 2015 Klaus Schliep, 2017 Franz Krah
+## Copyright 2002-2021 Emmanuel Paradis, 2015 Klaus Schliep, 2017 Franz Krah
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -135,7 +135,6 @@ as.list.DNAbin <- function(x, ...)
 }
 
 rbind.DNAbin <- function(...)
-### works only with matrices for the moment
 {
     obj <- list(...)
     n <- length(obj)
@@ -146,9 +145,8 @@ rbind.DNAbin <- function(...)
     NC <- unlist(lapply(obj, ncol))
     if (length(unique(NC)) > 1)
         stop("matrices do not have the same number of columns.")
-    for (i in 1:n) class(obj[[i]]) <- NULL
-    for (i in 2:n) obj[[1]] <- rbind(obj[[1]], obj[[i]])
-    structure(obj[[1]], class = "DNAbin")
+    for (i in 1:n) class(obj[[i]]) <- NULL # safe but maybe not really needed
+    structure(do.call(rbind, obj), class = "DNAbin")
 }
 
 cbind.DNAbin <-
@@ -264,7 +262,7 @@ print.DNAbin <- function(x, printlen = 6, digits = 3, ...)
         cat("Base composition:\n")
         print(round(base.freq(x), digits))
     } else {
-        cat("More than 10 million bases: not printing base composition\n")
+        cat("More than 10 million bases: not printing base composition.\n")
     }
     if (nTot > 1) {
         k <- floor(log(nTot, 1000))
@@ -383,15 +381,16 @@ Ftab <- function(x, y = NULL)
 
 GC.content <- function(x) sum(base.freq(x)[2:3])
 
-seg.sites <- function(x)
+seg.sites <- function(x, strict = FALSE, trailingGapsAsN = TRUE)
 {
     if (is.list(x)) x <- as.matrix(x)
     ## is.vector() returns FALSE because of the class,
     ## so we use a different test
     dx <- dim(x)
-    if (is.null(dx)) return(integer(0))
-    if (dx[1] == 1) return(integer(0))
-    ans <- .Call(SegSites, x)
+    if (is.null(dx)) return(integer())
+    if (dx[1] == 1) return(integer())
+    if (trailingGapsAsN) x <- latag2n(x)
+    ans <- .Call(SegSites, x, strict)
     which(as.logical(ans))
 }
 
@@ -416,12 +415,7 @@ dist.dna <- function(x, model = "K80", variance = FALSE, gamma = FALSE,
     }
     if (is.list(x)) x <- as.matrix(x)
     nms <- dimnames(x)[[1]]
-    n <- dim(x)
-    s <- n[2]
-    n <- n[1]
-
-    if (s * n > 2^31 - 1)
-        stop("dist.dna() cannot handle more than 2^31 - 1 bases")
+    n <- dim(x)[1] # in case nms is NULL
 
     if (imod %in% c(4, 6:8)) {
         BF <- if (is.null(base.freq)) base.freq(x) else base.freq
@@ -430,23 +424,21 @@ dist.dna <- function(x, model = "K80", variance = FALSE, gamma = FALSE,
     if (imod %in% 16:17) pairwise.deletion <- TRUE
 
     if (!pairwise.deletion) {
-        keep <- .C(GlobalDeletionDNA, x, n, s, rep(1L, s))[[4]]
+        keep <- .Call(GlobalDeletionDNA, x)
         x <- x[, as.logical(keep)]
-        s <- dim(x)[2]
     }
-    Ndist <- if (imod == 11) n*n else n*(n - 1)/2
-    var <- if (variance) double(Ndist) else 0
-    if (!gamma) gamma <- alpha <- 0
-    else {
+    if (!gamma) {
+        alpha <- 0
+    } else {
         alpha <- gamma
-        gamma <- 1
+        gamma <- 1L
+    }
+    d <- .Call(dist_dna, x, imod, BF, as.integer(pairwise.deletion),
+               as.integer(variance), as.integer(gamma), alpha)
+    if (variance) {
+        var <- d[[2]]
+        d <- d[[1]]
     }
-    d <- .C(dist_dna, x, as.integer(n), as.integer(s), imod,
-            double(Ndist), BF, as.integer(pairwise.deletion),
-            as.integer(variance), var, as.integer(gamma),
-            as.double(alpha), NAOK = TRUE)
-    if (variance) var <- d[[9]]
-    d <- d[[5]]
     if (imod == 11) {
         dim(d) <- c(n, n)
         dimnames(d) <- list(nms, nms)
@@ -566,12 +558,23 @@ alview <- function(x, file = "", uppercase = TRUE, showpos = TRUE)
 
 where <- function(x, pattern)
 {
-    pat <- as.DNAbin(strsplit(pattern, NULL)[[1]])
+    pat <- strsplit(pattern, NULL)[[1]]
+    if (inherits(x, "DNAbin")) {
+        pat <- as.DNAbin(pat)
+    } else {
+        if (inherits(x, "AAbin")) {
+            pat <- as.AAbin(toupper(pat))
+        } else {
+            stop("'x' should inherit class \"DNAbin\" or \"AAbin\"")
+        }
+    }
     p <- length(pat)
 
     f <- function(x, pat, p) {
-        s <- length(x)
-        if (s < p) stop("sequence shorter than the pattern")
+        if (length(x) < p) {
+            warning("sequence shorter than the pattern: returning NULL")
+            return(NULL)
+        }
         .Call(C_where, x, pat)
     }
 
@@ -657,8 +660,8 @@ trans <- function(x, code = 1, codonstart = 1)
     f <- function(x, s, code)
         .C(trans_DNA2AA, x, as.integer(s), raw(s/3), as.integer(code),
            NAOK = TRUE)[[3]]
-    if (code > 2)
-        stop("only the standard and the vertebrate mitochondrial codes are available for now")
+    if (code > 6)
+        stop("only the genetic codes 1--6 are available for now")
 
     if (codonstart > 1) {
         del <- -(1:(codonstart - 1))
@@ -681,7 +684,15 @@ trans <- function(x, code = 1, codonstart = 1)
             if (rest == 2) msg <- paste0(msg, "s")
             warning(paste(msg, "dropped"))
         }
-        res <- if (is.matrix(x)) t(apply(x, 1, f, s = s, code = code)) else f(x, s, code)
+        if (is.matrix(x)) {
+            res <- t(apply(x, 1, f, s = s, code = code))
+            if (s == 3) {
+                res <- t(res)
+                rownames(res) <- rownames(x)
+            }
+        } else {
+            res <- f(x, s, code)
+        }
     }
     class(res) <- "AAbin"
     res
@@ -728,10 +739,15 @@ print.AAbin <- function(x, ...)
 
 as.character.AAbin <- function(x, ...)
 {
-    f <- function(x) strsplit(rawToChar(x), "")[[1]]
-    if (is.list(x)) return(lapply(x, f))
-    if (is.matrix(x)) return(t(apply(x, 1, f)))
-    f(x)
+    f <- function(xx) {
+        ans <- strsplit(rawToChar(xx), "")[[1]]
+        if (is.matrix(xx)) {
+            dim(ans) <- dim(xx)
+            dimnames(ans) <- dimnames(xx)
+        }
+        ans
+    }
+    if (is.list(x)) lapply(x, f) else f(x)
 }
 
 as.AAbin <- function(x, ...) UseMethod("as.AAbin")
@@ -1091,6 +1107,71 @@ c.AAbin <- function(..., recursive = FALSE)
   structure(NextMethod("c"), class = "AAbin")
 }
 
+rbind.AAbin <- function(...)
+{
+    obj <- list(...)
+    n <- length(obj)
+    if (n == 1) return(obj[[1]])
+    for (i in 1:n)
+        if (!is.matrix(obj[[1]]))
+            stop("the 'rbind' method for \"AAbin\" accepts only matrices")
+    NC <- unlist(lapply(obj, ncol))
+    if (length(unique(NC)) > 1)
+        stop("matrices do not have the same number of columns.")
+    for (i in 1:n) class(obj[[i]]) <- NULL # safe but maybe not really needed
+    structure(do.call(rbind, obj), class = "AAbin")
+}
+
+cbind.AAbin <-
+    function(..., check.names = TRUE, fill.with.Xs = FALSE,
+             quiet = FALSE)
+{
+    obj <- list(...)
+    n <- length(obj)
+    if (n == 1) return(obj[[1]])
+    for (i in 1:n)
+        if (!is.matrix(obj[[1]]))
+            stop("the 'cbind' method for \"AAbin\" accepts only matrices")
+    NR <- unlist(lapply(obj, nrow))
+    for (i in 1:n) class(obj[[i]]) <- NULL
+    if (check.names) {
+        NMS <- lapply(obj, rownames)
+        for (i in 1:n)
+            if (anyDuplicated(NMS[[i]]))
+                stop("Duplicated rownames in matrix ", i, ": see ?cbind.AAbin")
+        nms <- unlist(NMS)
+        if (fill.with.Xs) {
+            NC <- unlist(lapply(obj, ncol))
+            nms <- unique(nms)
+            ans <- matrix(charToRaw("X"), length(nms), sum(NC))
+            rownames(ans) <- nms
+            from <- 1
+            for (i in 1:n) {
+                to <- from + NC[i] - 1
+                k <- match(NMS[[i]], nms)
+                ans[k, from:to] <- obj[[i]]
+                from <- to + 1
+            }
+        } else {
+            tab <- table(nms)
+            ubi <- tab == n
+            nms <- names(tab)[which(ubi)]
+            ans <- obj[[1]][nms, , drop = FALSE]
+            for (i in 2:n)
+                ans <- cbind(ans, obj[[i]][nms, , drop = FALSE])
+            if (!quiet && !all(ubi))
+                warning("some rows were dropped.")
+        }
+    } else {
+        if (length(unique(NR)) > 1)
+            stop("matrices do not have the same number of rows.")
+        ans <- matrix(unlist(obj), NR)
+        rownames(ans) <- rownames(obj[[1]])
+    }
+    class(ans) <- "AAbin"
+    ans
+}
+
 as.AAbin.list <- function(x, ...)
 {
   obj <- lapply(x, as.AAbin)
@@ -1151,109 +1232,55 @@ rDNAbin <- function(n, nrow, ncol, base.freq = rep(0.25, 4), prefix = "Ind_")
     res
 }
 
-dnds <- function(x, code = 1, codonstart = 1, quiet = FALSE)
+dnds <- function(x, code = 1, codonstart = 1, quiet = FALSE,
+                 details = FALSE, return.categories = FALSE)
 {
-    if (code > 2)
-        stop("only the standard (code=1) and the vertebrate mitochondrial (code=2) codes are available for now")
+    if (code > 6) stop("only the genetic codes 1--6 are available for now")
     if (is.list(x)) x <- as.matrix(x)
     n <- nrow(x)
     if (nrow(unique.matrix(x)) != n) stop("sequences are not unique")
+### if (any(base.freq(x, TRUE, TRUE)[-(1:4)] > 0)) stop("ambiguous bases are not permitted")
     if (codonstart > 1) {
         del <- -(1:(codonstart - 1))
         x <- x[, del]
     }
     p <- ncol(x)
     rest <- p %% 3
-    if (rest != 0) {
+    if (rest) {
         p <- p - rest
         x <- x[, 1:p]
-        msg <- paste("sequence length not a multiple of 3:",
-                     rest, ifelse(rest == 1,  "nucleotide", "nucleotides"),
-                     "dropped")
+        msg <- sprintf("sequence length not a multiple of 3: %d %s dropped",
+                       rest, ngettext(rest, "base", "bases"))
         warning(msg)
     }
 
-    class(x) <- NULL
-
-    abin <- as.raw(0x88)
-    gbin <- as.raw(0x48)
-    cbin <- as.raw(0x28)
-    tbin <- as.raw(0x18)
-
-    pos1 <- seq(1, by = 3, length.out = p/3)
-    POS1 <- POS2 <- POS3 <- logical(p)
-    POS1[pos1] <- POS2[pos1 + 1L] <- POS3[pos1 + 2L] <- TRUE
-
+    degMat <- .buildDegeneracyMatrix(code)
     Lcat <- matrix(0L, n, p)
-    Lcat[, POS3] <- 4L # most sites at 3rd positions are fourfold degenerate
+    V1 <- V2 <- V3 <- integer(136)
+    i <- c(136L, 72L, 40L, 24L)
+    V1[i] <- c(1L, 17L, 33L, 49L)
+    V2[i] <- c(0L, 4L, 8L, 12L)
+    V3[i] <- 0:3
 
-    m12 <- -c(1, 2)
+    class(x) <- NULL
+    z <- as.integer(x)
+    N <- length(x)
 
-    if (code == 1) {
+    SHIFT <- c(0L, n, 2L * n)
+    p <- 1L + SHIFT
+    while (p[3] <= N) {
         for (i in 1:n) {
-            z <- x[i, , drop = TRUE]
-            isA <- z == abin
-            isG <- z == gbin
-            isC <- z == cbin
-            isT <- z == tbin
-            ## 1/ are some bases at 1st positions twofold degenerate?
-            s2 <- isT & POS2
-            if (any(s2)) {
-                s3 <- c(FALSE, s2[-p]) & ((isA | isG) & POS3)
-                if (any(s3)) {
-                    s1 <- ((isT | isC) & POS1) & c(s3[m12], FALSE, FALSE)
-                    if (any(s1)) Lcat[i, which(s1)] <- 2L
-                }
-            }
-            s2 <- isG & POS2
-            if (any(s2)) {
-                s3 <- c(FALSE, s2[-p]) & ((isA | isG) & POS3)
-                if (any(s3)) {
-                    s1 <- ((isA | isC) & POS1) & c(s3[m12], FALSE, FALSE)
-                    if (any(s1)) Lcat[i, which(s1)] <- 2L
-                }
-            }
-            ## 2/ all bases at 2nd positions are nondegenerate: no need to do anything
-            ## 3/ find the bases at 3rd positions that are twofold degenerate
-            s2 <- isA & POS2
-            if (any(s2)) Lcat[i, which(s2) + 1L] <- 2L
-            s2 <- isG & POS2
-            if (any(s2)) {
-                s1 <- ((isT | isA) & POS1) & c(s2[-p], FALSE)
-                if (any(s1)) Lcat[i, which(s1) + 2L] <- 2L
-            }
-        }
-    } else { # if (code == 2)
-        for (i in 1:n) {
-            z <- x[i, , drop = TRUE]
-            isA <- z == abin
-            isG <- z == gbin
-            isC <- z == cbin
-            isT <- z == tbin
-            ## 1/ are some bases at 1st positions twofold degenerate?
-            s2 <- isT & POS2
-            if (any(s2)) {
-                s3 <- c(FALSE, s2[-p]) & ((isA | isG) & POS3)
-                if (any(s3)) {
-                    s1 <- ((isT | isC) & POS1) & c(s3[m12], FALSE, FALSE)
-                    if (any(s1)) Lcat[i, which(s1)] <- 2L
-                }
-            }
-            ## 2/ all bases at 2nd positions are nondegenerate: no need to do anything
-            ## 3/ find the bases at 3rd positions that twofold/fourfold degenerate
-            s1 <- (isA | isT) & POS1
-            if (any(s1)) {
-                s2 <- c(FALSE, s1[-p]) & ((isT | isA | isG) & POS2)
-                if (any(s2)) Lcat[i, which(s2) + 1L] <- 2L
-            }
-            s1 <- (isC | isG) & POS1
-            if (any(s1)) {
-                s2 <- c(FALSE, s1[-p]) & (isA & POS2)
-                if (any(s2)) Lcat[i, which(s2) + 1L] <- 2L
-            }
+            codon <- z[p]
+            ii <- V1[codon[1]] + V2[codon[2]] + V3[codon[3]]
+            if (!is.na(ii)) Lcat[p] <- degMat[ii, ]
+            p <- p + 1L
         }
+        p <- p[3] + SHIFT
     }
 
+    if (return.categories) return(Lcat)
+    if (details) quiet <- TRUE
+
     deg <- c(0, 2, 4) # the 3 levels of degeneracy
     nout <- n*(n - 1)/2
     res <- numeric(nout)
@@ -1263,12 +1290,23 @@ dnds <- function(x, code = 1, codonstart = 1, quiet = FALSE)
             if (!quiet) cat("\r", round(100*k/nout), "%")
             z <- x[c(i, j), ]
             Lavg <- (Lcat[i, ] + Lcat[j, ])/2
+            Lavg[Lavg == 1] <- 2
+            Lavg[Lavg == 3] <- 4
             ii <- lapply(deg, function(x) which(x == Lavg))
             L <- lengths(ii)
-            S <- lapply(ii, function(id) dist.dna(z[, id], "TS"))
-            V <- lapply(ii, function(id) dist.dna(z[, id], "TV"))
-            P <- unlist(S, use.names = FALSE)/L
-            Q <- unlist(V, use.names = FALSE)/L
+            S <- lapply(ii, function(id) dist.dna(z[, id, drop = FALSE], "TS"))
+            V <- lapply(ii, function(id) dist.dna(z[, id, drop = FALSE], "TV"))
+            S <- unlist(S, use.names = FALSE)
+            V <- unlist(V, use.names = FALSE)
+            if (details) {
+                cat(sprintf("\nComparing sequences %d and %d:\n", i, j))
+                tmp <- rbind(S, V)
+                dimnames(tmp) <- list(c("Transitions", "Transversions"),
+                                      c("Nondegenerate", "Twofold", "Fourfold"))
+                print(tmp)
+            }
+            P <- S/L
+            Q <- V/L
             a <- 1/(1 - 2*P - Q)
             b <- 1/(1 - 2*Q)
             c <- (a - b)/2
@@ -1289,3 +1327,82 @@ dnds <- function(x, code = 1, codonstart = 1, quiet = FALSE)
     class(res) <- "dist"
     res
 }
+
+.buildDegeneracyMatrix <- function(code) {
+    b <- as.raw(._bs_[1:4])
+    CODONS <- cbind(rep(b, each = 16), rep(rep(b, each = 4), 4), rep(b, 16))
+    AA <- trans(CODONS, code = code)
+
+    degeneracyMatrix <- matrix(0L, 64L, 3L)
+    deg <- c(4L, 2L, 2L, 0L)
+
+    ## 1/ find the bases at 3rd positions that are twofold/fourfold degenerate
+    s  <- 1:4
+    while (s[4L] <= 64) {
+        degeneracyMatrix[s, 3L] <- deg[length(unique(AA[s]))]
+        s <- s + 4L
+    }
+    ## 2/ all bases at 2nd positions are nondegenerate: no need to do anything
+    ## 3/ are some bases at 1st positions twofold degenerate?
+    s <- c(1L, 17L, 33L, 49L)
+    while (s[1L] < 17) {
+        degeneracyMatrix[s, 1L] <- deg[length(unique(AA[s]))]
+        s <- s + 1L
+    }
+    degeneracyMatrix
+}
+
+latag2n <- function(x) {
+    if (is.list(x)) x <- as.matrix(x)
+    dx <- dim(x)
+    clx <- class(x)
+    res <- .Call(leading_trailing_gaps_to_N, x)
+    class(res) <- clx
+    dim(res) <- dx
+    res
+}
+
+solveAmbiguousBases <- function(x, method = "columnwise", random = TRUE)
+{
+    if (method == "columnwise") {
+        if (is.list(x)) x <- as.matrix(x)
+        p <- ncol(x)
+        for (j in 1:p) {
+            BF <- base.freq(x[, j], TRUE, TRUE)
+            ambi <- BF[5:15]
+            K <- which(ambi > 0)
+            if (length(K)) {
+                agct <- BF[c(1, 3, 2, 4)]
+                for (b in K) {
+                    base <- as.DNAbin(names(ambi[b]))
+                    sel <- agct[rev(rawToBits(base))[1:4] == 1]
+                    if (!sum(sel)) sel[] <- 1L
+                    i <- which(x[, j] == base)
+                    tmp <- if (random) sample(names(sel), length(i), TRUE, sel) else names(sel)[which.max(sel)]
+                    x[i, j] <- as.DNAbin(tmp)
+                }
+            }
+        }
+    }
+    x
+}
+
+##distK80 <- function(x, pairwise.deletion = FALSE)
+##{
+##    nms <- dimnames(x)[[1]]
+##    n <- length(nms)
+##    if (!pairwise.deletion) {
+##        keep <- .Call(GlobalDeletionDNA, x)
+##        x <- x[, as.logical(keep)]
+##        d <- .Call(dist_dna_K80_short, x)
+##    } else {
+##        d <- .Call(dist_dna_K80_short_pairdel, x)
+##    }
+##    attr(d, "Size") <- n
+##    attr(d, "Labels") <- nms
+##    attr(d, "Diag") <- attr(d, "Upper") <- FALSE
+##    attr(d, "call") <- match.call()
+##    attr(d, "method") <- "K80"
+##    class(d) <- "dist"
+##    d
+##}
