@@ -1,8 +1,8 @@
-## ace.R (2017-04-25)
+## ace.R (2021-12-15)
 
 ##   Ancestral Character Estimation
 
-## Copyright 2005-2017 Emmanuel Paradis and Ben Bolker
+## Copyright 2005-2021 Emmanuel Paradis and 2005 Ben Bolker
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -137,12 +137,13 @@ ace <-
                 M <- dis[as.character(nb.tip + 1), MRCA]
                 dim(M) <- rep(sqrt(length(M)), 2)
             }
-            varAY <- M[-(1:nb.tip), 1:nb.tip]
-            varA <- M[-(1:nb.tip), -(1:nb.tip)]
-            V <- corMatrix(Initialize(corStruct, data.frame(x)),
-                           corr = FALSE)
+            one2n <- 1:nb.tip
+            varAY <- M[-one2n, one2n]
+            varA <- M[-one2n, -one2n]
+            DF <- data.frame(x)
+            V <- corMatrix(Initialize(corStruct, DF), corr = FALSE)
             invV <- solve(V)
-            o <- gls(x ~ 1, data.frame(x), correlation = corStruct)
+            o <- gls(x ~ 1, DF, correlation = corStruct)
             GM <- o$coefficients
             obj$ace <- drop(varAY %*% invV %*% (x - GM) + GM)
             names(obj$ace) <- (nb.tip + 1):(nb.tip + nb.node)
@@ -155,8 +156,7 @@ ace <-
     } else { # type == "discrete"
         if (method != "ML")
             stop("only ML estimation is possible for discrete characters.")
-        if (any(phy$edge.length <= 0))
-            stop("some branches have length zero or negative")
+        if (any(phy$edge.length < 0)) stop("some branches have negative length")
         if (!is.factor(x)) x <- factor(x)
         nl <- nlevels(x)
         lvls <- levels(x)
@@ -193,6 +193,7 @@ ace <-
         liks <- matrix(0, nb.tip + nb.node, nl)
         TIPS <- 1:nb.tip
         liks[cbind(TIPS, x)] <- 1
+        if (anyNA(x)) liks[which(is.na(x)), ] <- 1
         phy <- reorder(phy, "postorder")
 
         Q <- matrix(0, nl, nl)
@@ -222,7 +223,7 @@ ace <-
                     comp[anc] <- sum(v)
                     liks[anc, ] <- v/comp[anc]
                 }
-                if (output.liks) return(liks[-TIPS, ])
+                if (output.liks) return(liks[-TIPS, , drop = FALSE])
                 dev <- -2 * sum(log(comp[-TIPS]))
                 if (is.na(dev)) Inf else dev
             }
@@ -249,7 +250,7 @@ ace <-
                     comp[anc] <- sum(v)
                     liks[anc, ] <- v/comp[anc]
                 }
-                if (output.liks) return(liks[-TIPS, ])
+                if (output.liks) return(liks[-TIPS, , drop = FALSE])
                 dev <- -2 * sum(log(comp[-TIPS]))
                 if (is.na(dev)) Inf else dev
             }
@@ -292,6 +293,7 @@ ace <-
                     lik.anc <- lik.anc / rowSums(lik.anc)
                 }
             }
+            rownames(lik.anc) <- nb.tip + 1:nb.node
             colnames(lik.anc) <- lvls
             obj$lik.anc <- lik.anc
         }
